@@ -4,11 +4,20 @@ import random
 from collections import Counter, OrderedDict
 from copy import deepcopy
 from os.path import relpath
-from typing import Any, Dict, List, Optional, Set, Text, Tuple
+from typing import Any, Dict, List, Optional, Set, Text, Tuple, Callable
 
 import rasa.nlu.utils
 from rasa.utils.common import raise_warning, lazy_property
-from rasa.nlu.constants import RESPONSE, RESPONSE_KEY_ATTRIBUTE
+from rasa.nlu.constants import (
+    RESPONSE,
+    RESPONSE_KEY_ATTRIBUTE,
+    NO_ENTITY_TAG,
+    ENTITY_ATTRIBUTE_TYPE,
+    ENTITY_ATTRIBUTE_GROUP,
+    ENTITY_ATTRIBUTE_ROLE,
+    INTENT,
+    ENTITIES,
+)
 from rasa.nlu.training_data.message import Message
 from rasa.nlu.training_data.util import check_duplicate_synonym
 from rasa.nlu.utils import list_to_str
@@ -75,21 +84,35 @@ class TrainingData:
             nlg_stories,
         )
 
-    def filter_by_intent(self, intent: Text):
-        """Filter training examples """
+    def filter_training_examples(
+        self, condition: Callable[[Message], bool]
+    ) -> "TrainingData":
+        """Filter training examples.
 
-        training_examples = []
-        for ex in self.training_examples:
-            if ex.get("intent") == intent:
-                training_examples.append(ex)
+        Args:
+            condition: A function that will be applied to filter training examples.
+
+        Returns:
+            TrainingData: A TrainingData with filtered training examples.
+        """
 
         return TrainingData(
-            training_examples,
+            list(filter(condition, self.training_examples)),
             self.entity_synonyms,
             self.regex_features,
             self.lookup_tables,
         )
 
+    def filter_by_intent(self, intent: Text) -> "TrainingData":
+        """Filter training examples."""
+        raise_warning(
+            "The `filter_by_intent` function is deprecated. "
+            "Please use `filter_training_examples` instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self.filter_training_examples(lambda ex: intent == ex.get(INTENT))
+
     def __hash__(self) -> int:
         from rasa.core import utils as core_utils
 
@@ -102,52 +125,54 @@ class TrainingData:
     def sanitize_examples(examples: List[Message]) -> List[Message]:
         """Makes sure the training data is clean.
 
-        Remove trailing whitespaces from intent and response annotations and drop duplicate examples."""
+        Remove trailing whitespaces from intent and response annotations and drop
+        duplicate examples.
+        """
 
         for ex in examples:
-            if ex.get("intent"):
-                ex.set("intent", ex.get("intent").strip())
+            if ex.get(INTENT):
+                ex.set(INTENT, ex.get(INTENT).strip())
 
-            if ex.get("response"):
-                ex.set("response", ex.get("response").strip())
+            if ex.get(RESPONSE):
+                ex.set(RESPONSE, ex.get(RESPONSE).strip())
 
         return list(OrderedDict.fromkeys(examples))
 
     @lazy_property
     def intent_examples(self) -> List[Message]:
-        return [ex for ex in self.training_examples if ex.get("intent")]
+        return [ex for ex in self.training_examples if ex.get(INTENT)]
 
     @lazy_property
     def response_examples(self) -> List[Message]:
-        return [ex for ex in self.training_examples if ex.get("response")]
+        return [ex for ex in self.training_examples if ex.get(RESPONSE)]
 
     @lazy_property
     def entity_examples(self) -> List[Message]:
-        return [ex for ex in self.training_examples if ex.get("entities")]
+        return [ex for ex in self.training_examples if ex.get(ENTITIES)]
 
     @lazy_property
     def intents(self) -> Set[Text]:
         """Returns the set of intents in the training data."""
-        return {ex.get("intent") for ex in self.training_examples} - {None}
+        return {ex.get(INTENT) for ex in self.training_examples} - {None}
 
     @lazy_property
     def responses(self) -> Set[Text]:
         """Returns the set of responses in the training data."""
-        return {ex.get("response") for ex in self.training_examples} - {None}
+        return {ex.get(RESPONSE) for ex in self.training_examples} - {None}
 
     @lazy_property
     def retrieval_intents(self) -> Set[Text]:
         """Returns the total number of response types in the training data"""
         return {
-            ex.get("intent")
+            ex.get(INTENT)
             for ex in self.training_examples
-            if ex.get("response") is not None
+            if ex.get(RESPONSE) is not None
         }
 
     @lazy_property
     def examples_per_intent(self) -> Dict[Text, int]:
         """Calculates the number of examples per intent."""
-        intents = [ex.get("intent") for ex in self.training_examples]
+        intents = [ex.get(INTENT) for ex in self.training_examples]
         return dict(Counter(intents))
 
     @lazy_property
@@ -158,14 +183,55 @@ class TrainingData:
     @lazy_property
     def entities(self) -> Set[Text]:
         """Returns the set of entity types in the training data."""
-        entity_types = [e.get("entity") for e in self.sorted_entities()]
+        entity_types = [e.get(ENTITY_ATTRIBUTE_TYPE) for e in self.sorted_entities()]
         return set(entity_types)
 
+    @lazy_property
+    def entity_roles(self) -> Set[Text]:
+        """Returns the set of entity roles in the training data."""
+        entity_types = [
+            e.get(ENTITY_ATTRIBUTE_ROLE)
+            for e in self.sorted_entities()
+            if ENTITY_ATTRIBUTE_ROLE in e
+        ]
+        return set(entity_types) - {NO_ENTITY_TAG}
+
+    @lazy_property
+    def entity_groups(self) -> Set[Text]:
+        """Returns the set of entity groups in the training data."""
+        entity_types = [
+            e.get(ENTITY_ATTRIBUTE_GROUP)
+            for e in self.sorted_entities()
+            if ENTITY_ATTRIBUTE_GROUP in e
+        ]
+        return set(entity_types) - {NO_ENTITY_TAG}
+
+    def entity_roles_groups_used(self) -> bool:
+        entity_groups_used = (
+            self.entity_groups is not None and len(self.entity_groups) > 0
+        )
+        entity_roles_used = self.entity_roles is not None and len(self.entity_roles) > 0
+
+        return entity_groups_used or entity_roles_used
+
     @lazy_property
     def examples_per_entity(self) -> Dict[Text, int]:
         """Calculates the number of examples per entity."""
-        entity_types = [e.get("entity") for e in self.sorted_entities()]
-        return dict(Counter(entity_types))
+
+        entities = []
+
+        def _append_entity(entity: Dict[Text, Any], attribute: Text) -> None:
+            if attribute in entity:
+                _value = entity.get(attribute)
+                if _value is not None and _value != NO_ENTITY_TAG:
+                    entities.append(f"{attribute} '{_value}'")
+
+        for entity in self.sorted_entities():
+            _append_entity(entity, ENTITY_ATTRIBUTE_TYPE)
+            _append_entity(entity, ENTITY_ATTRIBUTE_ROLE)
+            _append_entity(entity, ENTITY_ATTRIBUTE_GROUP)
+
+        return dict(Counter(entities))
 
     def sort_regex_features(self) -> None:
         """Sorts regex features lexicographically by name+pattern"""
@@ -177,8 +243,8 @@ class TrainingData:
         """Set response phrase for all examples by looking up NLG stories"""
         for example in self.training_examples:
             response_key = example.get(RESPONSE_KEY_ATTRIBUTE)
-            # if response_key is None, that means the corresponding intent is not a retrieval intent
-            # and hence no response text needs to be fetched.
+            # if response_key is None, that means the corresponding intent is not a
+            # retrieval intent and hence no response text needs to be fetched.
             # If response_key is set, fetch the corresponding response text
             if response_key:
                 # look for corresponding bot utterance
@@ -299,7 +365,7 @@ class TrainingData:
         """Sorts the intent examples by the name of the intent and then response"""
 
         return sorted(
-            self.intent_examples, key=lambda e: (e.get("intent"), e.get("response"))
+            self.intent_examples, key=lambda e: (e.get(INTENT), e.get(RESPONSE))
         )
 
     def validate(self) -> None:
@@ -331,10 +397,10 @@ class TrainingData:
                 )
 
         # emit warnings for entities with only a few training samples
-        for entity_type, count in self.examples_per_entity.items():
+        for entity, count in self.examples_per_entity.items():
             if count < self.MIN_EXAMPLES_PER_ENTITY:
                 raise_warning(
-                    f"Entity '{entity_type}' has only {count} training examples! "
+                    f"Entity {entity} has only {count} training examples! "
                     f"The minimum is {self.MIN_EXAMPLES_PER_ENTITY}, because of "
                     f"this the training may fail."
                 )
@@ -393,7 +459,7 @@ class TrainingData:
     ) -> Tuple[list, list]:
         train, test = [], []
         for intent, count in self.examples_per_intent.items():
-            ex = [e for e in self.intent_examples if e.data["intent"] == intent]
+            ex = [e for e in self.intent_examples if e.data[INTENT] == intent]
             if random_seed is not None:
                 random.Random(random_seed).shuffle(ex)
             else:
@@ -405,20 +471,27 @@ class TrainingData:
         return test, train
 
     def print_stats(self) -> None:
+        logger.info("Training data stats:")
         logger.info(
-            "Training data stats: \n"
-            + "\t- intent examples: {} ({} distinct intents)\n".format(
-                len(self.intent_examples), len(self.intents)
-            )
-            + "\t- Found intents: {}\n".format(list_to_str(self.intents))
-            + "\t- Number of response examples: {} ({} distinct response)\n".format(
-                len(self.response_examples), len(self.responses)
-            )
-            + "\t- entity examples: {} ({} distinct entities)\n".format(
-                len(self.entity_examples), len(self.entities)
-            )
-            + "\t- found entities: {}\n".format(list_to_str(self.entities))
+            f"Number of intent examples: {len(self.intent_examples)} "
+            f"({len(self.intents)} distinct intents)"
         )
+        if self.intents:
+            logger.info(f"  Found intents: {list_to_str(self.intents)}")
+        logger.info(
+            f"Number of response examples: {len(self.response_examples)} "
+            f"({len(self.responses)} distinct responses)"
+        )
+        logger.info(
+            f"Number of entity examples: {len(self.entity_examples)} "
+            f"({len(self.entities)} distinct entities)"
+        )
+        if self.entities:
+            logger.info(f"  Found entity types: {list_to_str(self.entities)}")
+        if self.entity_roles:
+            logger.info(f"  Found entity roles: {list_to_str(self.entity_roles)}")
+        if self.entity_groups:
+            logger.info(f"  Found entity groups: {list_to_str(self.entity_groups)}")
 
     def is_empty(self) -> bool:
         """Checks if any training data was loaded."""
